VERSION 5.00
Begin VB.UserControl TTOSock 
   BorderStyle     =   1  'Fixed Single
   CanGetFocus     =   0   'False
   ClientHeight    =   816
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   1140
   InvisibleAtRuntime=   -1  'True
   Picture         =   "TTOSock.ctx":0000
   ScaleHeight     =   816
   ScaleWidth      =   1140
End
Attribute VB_Name = "TTOSock"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private ListeningOnSocket As Long
Private MyUCKey As String

'********************************************************************
'
'                              Events
'
'********************************************************************

Public Event ConnectionRequest()
Public Event Connected(ByVal SocketID As Long)
Public Event DataArrival(ByVal SocketID As Long, sData As String)
Public Event Error(ByVal SocketID As Long, ByVal number As Integer, Description As String)
Public Event ConnectionsAlert(ByVal SocketID As Long)
Public Event PeerClosing(ByVal SocketID As Long)
Public Event SendComplete(ByVal SocketID As Long)
Public Event StateChanged(ByVal SocketID As Long)
'

'********************************************************************
'
'                           Properties
'
'********************************************************************

'Returns the Current State of the specified Socket
Public Property Get State(SocketID As Long) As Integer

  Dim x As Integer
  'Checks for the index in m_lngSocks() array. If the socket does not
  'exist, -1 is returned.
  x = GetIndexFromsID(SocketID)
  
  'If the socket does not exist, return -1.
  'Otherwise, return the current state of the socket.
  If x = -1 Then
    State = -1
  Else
    State = CurrentState(x)
  End If

End Property

'Since each socket array can only handle up to 64 simultaneous
'event calls per thread to WindowProc(), you can raise an event
'when you reach a specified number of connections. By default,
'the event will raise at 50 open sockets. This limitation is
'in the Winsock API.
Public Property Let MaxConnectionsAlert(v_intMaxConnectionsAlert As Integer)

  m_intConnectionsAlert = v_intMaxConnectionsAlert

End Property

'Returns the Max Connections Alert Value
Public Property Get MaxConnectionsAlert() As Integer

  MaxConnectionsAlert = m_intConnectionsAlert

End Property

Public Property Get ListeningSocket() As Long

  ListeningSocket = ListeningOnSocket

End Property

'********************************************************************
'
'                              Methods
'
'********************************************************************

'This function allows you to Accept a connect from a listening socket,
'spawn a new Socket, and connect using this new socket.
Public Function AcceptConnection() As Long
  
  'I'm not sure why RC is used in the Winsock API as the error checking
  'variable, but it generally is, so I used it too.
  Dim RC As Long
  Dim i As Integer
  Dim ReadSockBuffer As sockaddr
    
  'Gets the next available index in the m_lngSocks() array.
  i = GetNextSocksIndex
  m_lngSocks(i) = accept(ListeningOnSocket, ReadSockBuffer, Len(ReadSockBuffer))
  
  'If -1 is returned, then an error has occurred in the API accept
  'routine. If -1 is not returned, then the number of the new
  'socket is returned.
  If m_lngSocks(i) = -1 Then
    'Handle Error
    Exit Function
  End If
  
  'This sets the Asyncronous values to raise an event on.
  'See the Readme module for more on this.
  RC = WSAAsyncSelect(m_lngSocks(i), hWnd, ByVal (1025 + Int(Right$(MyUCKey, Len(MyUCKey) - 1))), ByVal FD_READ Or FD_CLOSE Or FD_WRITE Or FD_OOB Or FD_CONNECT)
        
  'If -1 is returned, then an error has occurred in the API
  'WSAAsyncSelect routine. Another value is acceptable.
  If RC = -1 Then
    'Handle Error
    Exit Function
  End If
    
  'For some reason, FD_CONNECT is not being raised in the WindowProc
  'fucntion, so I am raising here, after the accept has been completed.
  'I have a feeling that it may be because:
  '1) We do not accept connections on the listening socket, the
  '    accept connection is passed off to another socket;
  '2) Therefore, by setting the WSAAsyncSelect AFTER (which is when we
  '    are forced to set it) the accept, we are missing the connect
  '    event all together;
  '3) I will be treating this as a bug, and looking for a resolution to
  '    it.
  RaiseEvent Connected(m_lngSocks(i))
    
  'Return the number of the new socket
  AcceptConnection = m_lngSocks(i)
    
End Function

'Destroys the connection and references to this SocketID
Public Function Disconnect(SocketID As Long)

  'Since a user could possibly attempt to destroy a socket that has
  'already been destroyed, we need to keep chugging along with the
  'routine.
  On Error Resume Next
  Dim x As Integer
  
  'This is an API call to close the socket
  closesocket CInt(SocketID)
  
  'Remove Array Reference
  x = GetIndexFromsID(SocketID)
  m_lngSocks(x) = -1
      
End Function

'Returns the local IP being used. If the socket is
'listening, the IP on which we are listing is returned.
'If the SocketID does not exist, "0.0.0.0" is returned.
Public Function LocalIP(SocketID As Long) As String
  
  Dim x As String
  Dim Y As Integer
    
  'If the socket does not exist, return 0.0.0.0
  If Not IDExists(SocketID) Then
    LocalIP = "0.0.0.0"
    Exit Function
  End If
  
  'The peer address function returns the IP and the port in
  'one string, where the format of the string is:
  '      IP        Port
  '###.###.###.###:####
  'We just have to strip away the port.
  x = GetSockAddress(Int(SocketID))
  
  Y = InStr(1, x, ":")
  x = Left$(x, Y - 1)

  LocalIP = x
    
End Function

'Returns the local port being used. If the SocketID is connected
'to another IP, the port in use is returned. If the socket is
'listening, the port on which we are listing is returned.
'If the SocketID does not exist, -1 is returned.
Public Function LocalPort(SocketID As Long) As Long

  Dim x As String
  Dim Y As Integer
  
  'If the socket does not exist, return 0.0.0.0
  If Not IDExists(SocketID) Then
    LocalPort = 0
    Exit Function
  End If
    
  'The peer address function returns the IP and the port in
  'one string, where the format of the string is:
  '      IP        Port
  '###.###.###.###:####
  'We just have to strip away the port.
  x = GetSockAddress(Int(SocketID))

  Y = InStr(1, x, ":")
  
  x = Right$(x, Len(x) - Y)

  LocalPort = CLng(x)

End Function

'This function opens a port to listen on, as defined in the
'HostPort property
Public Function ListenNow(OnPort As Long) As Boolean
  
  Dim RC As Long
  Dim SocketBuffer As sockaddr
  
  'When we listen on a socket, we need to tell the OS to call a
  'procedure that will act as our event handler. I'm not sure
  'exactly what the function returns, but I have seen that it
  'is used in a variety of projects, so I used it too.
  'hWnd - The handle of the usercontrol.
  'GWL_WNDPROC - Not too sure on this constant.
  'AddressOf WindowProg - returns the memory location of the
  'function that we will use to handle our events. This function
  'is in the API_Declarations module.
  OldWndProc = SetWindowLong(UserControl.hWnd, GWL_WNDPROC, AddressOf WindowProc)
  
  ListeningOnSocket = Socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)
    
  'If there was an error ListeningOnSocket would contain the value
  '-1, otherwise it would contain the value of the socket handle.
  'We have not established that the socket will be used to listen
  'on yet, we have simply aquired a new socket.
  If ListeningOnSocket = -1 Then
    'Handle Error
    Exit Function
  End If
    
  'This defines several variable for the a new listening socket,
  'As per the Winsock API Type Definition
  SocketBuffer.sin_family = AF_INET
  SocketBuffer.sin_port = htons(OnPort)
  SocketBuffer.sin_addr = 0
  SocketBuffer.sin_zero = String$(8, 0)
    
  'Here we bind the newly aquired socket with the data that we
  'have defined in SocketBuffer
  RC = bind(ListeningOnSocket, SocketBuffer, 16)
  
  'If RC = 0, the bind was successful. If any other value was
  'returned, the bind was a bust.
  If RC <> 0 Then
    'Deal with Error
    ListenNow = False
    Exit Function
  End If
    
  'Attempt to listen on the socket. The 5 specifies the maximum
  'number of connections to hold in the queue. That is, if 7 sockets
  'attempt to connect to our listening socket over say 1 second, and
  'our program does not accept the connections (for whatever reason)
  'we will reject the last two attempts to connect to us.
  'This value can be defined from 1 to 5. Any other value is
  'rounded to the nearest valid integer.
  'Also, I don't think that this value has any bearing in a
  'Windows 9x environment. I think that it is always 5, regardless
  'of what we define it as.
  RC = listen(ByVal ListeningOnSocket, ByVal 5)
  
  'If RC = 0, the bind was successful. If any other value was
  'returned, the listen was a bust.
  If RC <> 0 Then
    'Handle Errors
    ListenNow = False
    Exit Function
  End If
  
  'This sets the Asyncronous values to raise an event on.
  'See the Readme module for more on this.
  RC = WSAAsyncSelect(ListeningOnSocket, hWnd, ByVal (1025 + Int(Right$(MyUCKey, Len(MyUCKey) - 1))), ByVal FD_CONNECT Or FD_ACCEPT)
  If RC <> 0 Then
    'Handle Errors
    ListenNow = False
    Exit Function
  End If

  ListenNow = True
  
End Function

'This function returns the remote IP of the socket that we
'specify.
Public Function RemoteIP(SocketID As Long) As String

  Dim x As String
  Dim Y As Integer
    
  'If the socket does not exist, return 0.0.0.0
  If Not IDExists(SocketID) Then
    RemoteIP = "0.0.0.0"
    Exit Function
  End If
  
  'The peer address function returns the IP and the port in
  'one string, where the format of the string is:
  '      IP        Port
  '###.###.###.###:####
  'We just have to strip away the port.
  x = GetPeerAddress(Int(SocketID))
  
  Y = InStr(1, x, ":")
  x = Left$(x, Y - 1)

  RemoteIP = x

End Function

'This function returns the remote port of the socket that we
'specify.
Public Function RemotePort(SocketID As Long) As Long

  Dim x As String
  Dim Y As Integer
  
  'If the socket does not exist, return 0.0.0.0
  If Not IDExists(SocketID) Then
    RemotePort = 0
    Exit Function
  End If
    
  'The peer address function returns the IP and the port in
  'one string, where the format of the string is:
  '      IP        Port
  '###.###.###.###:####
  'We just have to strip away the port.
  x = GetPeerAddress(Int(SocketID))

  Y = InStr(1, x, ":")
  
  x = Right$(x, Len(x) - Y)

  RemotePort = CLng(x)

End Function

'This function sends data out to the specified socket. If no
'socket is specified (or if socket 0 is specified) the data
'is sent to all sockets.
Public Function SendDataTo(DataToSend As String, Optional SocketID As Long = 0)

  If SocketID = 0 Then
    
    Dim x As Integer
    
    'Sends the data to each socket
    For x = 1 To m_intMaxSockCount
      If m_lngSocks(x) <> -1 Then SendToSock DataToSend, m_lngSocks(x)
    Next x
    
  Else
    
    'If the socket does not exist, get out
    If Not IDExists(SocketID) Then
      Exit Function
    End If
    
    SendToSock DataToSend, SocketID
    
  End If

End Function

'This returns a description of the state specified, based on socket,
'and state specified
Public Function StateDescription(Optional SocketID As Long = 0, Optional v_intSelState As Integer = -1) As String

  Dim x As Integer
  
  'If the state specifed is invalid (not 0-9), do this
  If v_intSelState < 0 Or v_intSelState > 9 Then
     
    'Get the SocketID stack index, based on the socket id
    x = GetIndexFromsID(SocketID)
  
    'If x = -1, the socked id does not exist.
    If x = -1 Then
      'Return an error.
      StateDescription = "Socket Does Not Exist"
    Else
      'Return the state of the socket
      StateDescription = WinsockStates(CurrentState(x))
    End If
    
  Else
    'If the state specified is valid (0-9), return its description
    StateDescription = WinsockStates(v_intSelState)
  End If

End Function

'********************************************************************
'
'                           More Methods
'                          ^^^^^^^^^^^^^^
'    These subs were added  to allow  the API_Declarations Module
'    access  to the  usercontrol's  events.  As such, the  events
'    are  also exposed  to the control's  user. If they  are  not
'    abused, this should be fine.
'
'********************************************************************

Public Sub RaiseConnected(ByVal SocketID As Long)

  RaiseEvent Connected(SocketID)

End Sub

Public Sub RaiseConnectionRequest()

  RaiseEvent ConnectionRequest

End Sub

Public Sub RaiseDataArrival(ByVal SocketID As Long, sData As String)

  RaiseEvent DataArrival(SocketID, sData)

End Sub

Public Sub RaiseError(ByVal SocketID As Long, ByVal number As Integer, Description As String)

  RaiseEvent Error(SocketID, number, Description)

End Sub

Public Sub RaisePeerClosing(ByVal SocketID As Long)

  RaiseEvent PeerClosing(SocketID)

End Sub

Public Sub RaiseSendComplete(ByVal SocketID As Long)

  RaiseEvent SendComplete(SocketID)

End Sub

Public Sub RaiseStateChanged(ByVal SocketID As Long)

  RaiseEvent StateChanged(SocketID)

End Sub

'********************************************************************
'
'                    Private Functions and Subs
'
'********************************************************************

'This function returns an empty index in the socket stack. If there
'is no free socket index in the stack, a new entry is added.
Private Function GetNextSocksIndex() As Integer

  Dim x As Integer
  
  'Look through the stack and check for a socket ID that = -1
  For x = 1 To m_intMaxSockCount
    If m_lngSocks(x) = -1 Then
      'If we found one, return it's index and exit
      GetNextSocksIndex = x
      Exit Function
    End If
  Next x
  
  'If we did not find a free index in the socket stack, add new
  'space at the end of the arrays, and set their initial values.
  m_intMaxSockCount = m_intMaxSockCount + 1
  
  ReDim Preserve m_lngSocks(m_intMaxSockCount)
  ReDim Preserve CurrentState(m_intMaxSockCount)
  
  m_lngSocks(m_intMaxSockCount) = -1
  CurrentState(m_intMaxSockCount) = -1
  
  GetNextSocksIndex = m_intMaxSockCount
  
End Function

'This function checks to see if the SocketID exists in the
'socket stack.
Private Function IDExists(SocketID As Long) As Boolean

  Dim x As Integer
  
  For x = 1 To m_intMaxSockCount
    If m_lngSocks(x) = SocketID Then
      IDExists = True
      Exit Function
    End If
  Next x
  
  IDExists = False

End Function

'This function sends data to a specified socket
Private Sub SendToSock(msg As String, SocketID As Long)

  On Error Resume Next
  Dim RC As Long
    
  RC = SendData(SocketID, msg)
        
  'If RC = -1, then the data could not be sent.
  If RC = -1 Then
    'Handle Error
  Else
    RaiseEvent SendComplete(SocketID)
  End If
  
End Sub

'********************************************************************
'
'                         UserControl Events
'
'********************************************************************

Private Sub UserControl_Initialize()

  On Error Resume Next

  If Not WinsockStartedUp Then

    Dim RC As Long
    Dim StartupData As WSADataType

    'StartupData is returned to us. It contains various notes about
    'our system and the version of Winsock that we are using.
    RC = WSAStartup(&H101, StartupData)
  
    If RC = -1 Then
      'Handle Error
      Exit Sub
    End If
    
    WinsockStartedUp = True
    
    WinsockStates(0) = "Closed"
    WinsockStates(1) = "Open"
    WinsockStates(2) = "Listening"
    WinsockStates(3) = "Connection Pending"
    WinsockStates(4) = "Resolving Host"
    WinsockStates(5) = "Host Resolved"
    WinsockStates(6) = "Connecting"
    WinsockStates(7) = "Connected"
    WinsockStates(8) = "Peer Is Closing The Connection"
    WinsockStates(9) = "Error On Socket"
      
    m_intMaxSockCount = 1
    ReDim Preserve m_lngSocks(1)
    ReDim Preserve CurrentState(1)
    m_lngSocks(1) = -1
    CurrentState(1) = -1
    
  End If

  'This function allows the API_Declarations to acces methods and
  'properties of this user control.
  MyUCKey = SetControlHost(Me)
  'MsgBox MyUCKey
  'Default value of when to trigger the ConnectionsAlert Event.
  m_intConnectionsAlert = 50

End Sub

Private Sub UserControl_Terminate()

  CleanUp MyUCKey
  
  On Error Resume Next
  
  Dim RC As Long
  Dim x As Integer
  
  'Closes any open sockets
  For x = 1 To m_intMaxSockCount
      
    closesocket CInt(m_lngSocks(x))
      
  Next x
  
  closesocket CInt(ListeningOnSocket)
  'Close winsock and clean it up.
  RC = WSCleanUp()

End Sub

'This just forces the usercontrol to remain a constant size
Private Sub UserControl_Resize()

  UserControl.Width = 450
  UserControl.Height = 430

End Sub

